/* -*- mode: c++; c-basic-offset: 3; -*- */
#ifndef WPIPE_WSTRCHAN_HH
#define WPIPE_WSTRCHAN_HH

#include "wtypes.hh"
#include "weventlist.hh"
#include "wparameters.hh"
#include "wtile.hh"
#include "streaming_conditioner.hh"
#include "thread_base.hh"
#include <vector>

class MultiDacc;
class OperStateCondList;

namespace wpipe {
   class woutput;

   /**  The %wstream_chan class defines a processes a group of channels that
     *  are to be processed in an identical manner and with a common tiling. 
     *  Any channels that are to be analysed coherently (not implemented yet)
     *  must be in the same group.
     *  \brief Define a channel group for identical processing.
     *  \author John Zweizig <john.zweizig@ligo.org>
     */
   class wstream_chan : public thread::thread_base
   {
   public:
      /**  Construct a %wstream_chan instance. The channel names, analysis 
        *  parameters, etc. are specified in the wparameters instance passed
	*  to the constructor.
	*  \param par Channel names, sources and analysis parameters for 
	*             this channel group.
	*  \param debugLevel Debugging printout level.
	*/
      wstream_chan(const wparameters& par, int dbugLvl);

      /**  Destroy the wstream channel group descriptor.
 	*/ 
      ~wstream_chan(void);
      
      /** Add a channel to the dacc list.
       */
      void addChannels(MultiDacc& md);
      void addOutputFiles(woutput& wo, const std::string& fileTime, 
			  const std::string& outdir) const;

      /**  Book a channel in the appropriate %MultiDacc stream. THe frame type 
        *  argument is used to select an input stream. I there is only only one
	*  stream the channel will be book to that stream regardless of the 
	*  stream name or frame type. If there is more than one stream, the 
	*  site name and frame type (assuming a name of the form 
	*  \c \<dir>/\<site>-\<type>-\* ) of the first file in the stream 
	*  input index is compared to \a frameType. If they are the same, 
	*  the channel is booked in that steam. If no streams math the 
	*  \a frameType argument, the first stream is used. 
        *  \brief Book a %MultiDacc input channel.
        *  \param md        MultiDacc input device.
	*  \param channel   chanel name.
	*  \param frameType frame type string.
	*/
      void bookInput(MultiDacc& md, const std::string& channel,
		     const std::string& frameType) const;

      void inject(MultiDacc& md);
      double minimumOverlap(void) const;
      size_t numberOfChannels(void) const;

      /**  Prepare the signal conditioning filters with a short span of data.
       */
      void prepare(void);

      /**  Read data from all channels in this group. Condition the incremental
        *  data segment, perform a q-transform and write out selected events.
	*  \brief process data.
	*  \param wo             Output file definer.
	*  \param blockStartTime Start time of data to be processed.
	*  \param blockDuration  Duration of data to be processed.
	*  \param blockOverlap   Overlap duration for succesive data blocks.
	*/ 
      void process(const woutput& wo, const Time& blockStartTime, 
		   Interval blockDuration, Interval blockOverlap);

      /**  Read in state data for times from start - start+dT and select 
        *  channels that should have good data. Read data from the good 
	*  channels for the specified times. On return, validChannels 
	*  indicates channels that pass the state criteria and input data 
	*  for the valid channels are available in input_data[].
	*/
      void read_data(MultiDacc& md, const Time& start, double dT);

      /**  Settling time needed by signal conditioning (resampling, high-pass)
        *  filters. The settling time is assumed to be symmetric so that the 
	*  conditioned data intervals of length settle_time() are invalid from
	*  the start of processed data and at the end of processed data. 
	*  The settling time is rounded up to a multple of a full second. 
	*  \returns settling time required by signal conditioning filters.
	*/
      Interval settle_time(void) const;

      /**  This method is called by read_data to validate the channels to be
        *  processed during the specified time span using OSC state information.
	*  The validChannels flag is cleared for each channel with an invalid 
	*  OSC condition.
        *  \brief Test the OSC state conditions requested.
	*  \param md    Multi-frame reader with data from the current span.
	*  \param start Start of the time span to be evaluated.
	*  \param dT    duration of the time span to be evaluated.
	*/
      void test_osc(MultiDacc& md, const Time& start, double dT);

      /**  This method is called by read_data to validate the channels to be
        *  processed during the specified time span. The validChannels flag
	*  is cleared for each channel with an invalid state condition.
        *  \brief Test the state condition bits requested.
	*  \param md    Multi-frame reader with data from the current span.
	*  \param start Start of the time span to be evaluated.
	*  \param dT    duration of the time span to be evaluated.
	*/
      void test_state(MultiDacc& md, const Time& start, double dT);

      /**  Perform threaded processing.
	*/ 
      void* thread_entry(void);

      const wparameters& refParameters(void) const;
      const wtile& refTiling(void) const;

      /**  reset conditioners and input, state, osc tseries vectors. Call 
        *  start_chunk to reset valid data information.
	*/
      void reset(void);

      /**  Make sure all validity check data are constructed. Reset channel 
        *  validity check information.
	*/
      void start_chunk(void);
      weventstack& trigClusters(void);
      weventstack& trigDownselect(void);
      void validChans(str_vect& processedChannels) const;

   private:
      wparameters pars;
      wtile       tiling;
      int         debugLevel;
      double      blockOverlap;
      double      minimumTimeShift;
      double      maximumTimeShift;
      
      std::vector<streaming_conditioner> conditioner;

      tser_vect   input_data;
      tser_vect   state_data;
      
       /**  The state_index vector indicates which states need to be evaluated 
        *  independently. If state_index < 0, the state is always true, \e i.e.
	*  the state channel name was empty or "NONE" or the mask was zero.
	*  If \c state_index>=0 it indicates the first channel with an identical
	*  state. The state need be evaluated only for states where 
	*  state_index[i] == i
	*/ 
      std::vector<int> state_index;

      /**  OSC condition test list.
       */
      OperStateCondList* oscList;
      std::vector<int> osc_index;
      tser_vect   osc_data;

      /**  validChannels is a bit vector that is true if all previous processing
        *  of the corresponding channel has succeeded.
	*/
      bool_vect   validChannels;
      weventstack triggersDownselected;
      weventstack clusters;
#ifndef NO_COINCIDENCE
      weventstack event;
#endif
   };

   typedef std::vector<wstream_chan> strchan_vect;

   //===================================  Inline methods
   inline size_t
   wstream_chan::numberOfChannels(void) const {
      return pars.numberOfChannels();
   }

   inline const wparameters& 
   wstream_chan::refParameters(void) const {
      return pars;
   }

   inline const wtile& 
   wstream_chan::refTiling(void) const {
      return tiling;
   }

   inline weventstack& 
   wstream_chan::trigClusters(void) {
      return clusters;
   }

   inline weventstack& 
   wstream_chan::trigDownselect(void) {
      return triggersDownselected;
   }
}
#endif // !defined(WPIPE_WSTRCHAN_HH)
