/* -*- mode: c++; c-basic-offset: 3; -*- */
#ifndef Coherence_Monitor_HH
#define Coherence_Monitor_HH

// startincludes
#include "DatEnv.hh"
#include "MonServer.hh"
#include "FixedLenTS.hh"
#include "Trend.hh"
#include "TrigClient.hh"
#include "Coherence.hh"
#include "OperStateCondList.hh"
#include "autopipe.hh"
#include "multiframe_writer.hh"
#include "usertime.hh"
#include <string>
// endincludes

//
//    Data Monitor template class.

/**  @name Coherence_Monitor
 *  @memo Template of a data environment (DatEnv) based monitor.
 *  The Coherence_Monitor monitor is a template for a data monitor program.
 *  Coherence_Monitor makes a simple calculation of the standard deviation of
 *  a specified channel (by default H0:PEM-LVEA_SEISX), serves the result 
 *  for display by the DMT viewer, stores the result as a minute trend and
 *  optionally writes a segment.
 *
 *  {\bf {\Large Running Coherence_Monitor}}
 *
 *  The Coherence_Monitor command line syntax is:
 *  \begin{verbatim}
 Coherence_Monitor [-channel <channel>] [-frames <number>] [-stride <time>] \
 [-filter <filter>] [-settle <s-time>] 
 \end{verbatim}
 *  The \c \<number> argument specifies the number of data strides to process
 *  before terminating the program. The \c \<time> argument specifies the data
 *  stride.
 *
 *  {\bf {\Large Configuration File}}
 *
 *  Coherence_Monitor uses a fixed configuration. There is therfore no 
 *  configuration file
 *
 *  {\bf {\Large Coherence_Monitor Output}}
 *
 *  {\bf Trend channels}
 *  Coherence_Monitor produces a minute trend of the sigma value called 
 *  #<channel>_sigma#
 *
 *  {\bf Monitor data channels}
 *  Coherence_Monitor publishes a time series history of the sigma values 
 *  with the name #<channel>_sigma#
 *
 *  @author John Zweizig; Caltech/LIGO
 *  @version 2.0; Last modified July 26, 2020
 */
//{
//}

// classdef
class Coherence_Monitor : public DatEnv, MonServer 
{
public:
   /** Coherence monitor constructor.
    */
   Coherence_Monitor(int argc, const char *argv[]);

   /** Coherence monitor termination method / destructor.
    */
   ~Coherence_Monitor(void);

   /** Configuration method.
    */
   void configure(const std::string& file);
   /** Process a data chunk 
    */
   void ProcessData(void);
   /** Handle Message signal.
    */
   void Attention(void);

   /** Print out the configuration.
    * \brief Display configuration.
    * \param out Output stream to receive the configuration information.
    */
   void display(std::ostream& out) const;

   /** print results after on stride.
    */
   void print(void) const;
   
   /** Reset a averaging segment
    */
   void reset(void);

   /** Write the data to frames.
    */
   void write(const Time& t0, Interval dt);

private:
   class coherence_channel {
   public:
      coherence_channel(const std::string& chan, const calc_dft& co);
      void add(const TSeries& ts);
      const std::string& alias(void) const;
      const std::string& channel(void) const;
      double fmax(void) const;
      const containers::DFT& last_DFT(void) const;
      void next_stride(const containers::DFT& ref);
      const Coherence& ref_coherence(void) const;
      Coherence& ref_coherence(void);
      const containers::CSD& refCSD(void) const;
      const containers::PSD& refPSD(void) const;
      void reset(void);
      void set_alias(const std::string& nick);
      void set_debug(int lvl);
      void set_fmax(double f);
      Interval stride(void) const;
      std::string window_type(void) const;
   private:
      std::string mChannel;    ///< coherence channel name
      std::string mAlias;      ///< channel alias (unique short name)
      double      mFmax;       ///< maximum frequency to plot coherence.
      int         mDebug;      ///< debug level.
      Coherence   mCoherence;  ///< coherence tool.
   };
   typedef std::vector<coherence_channel>          chan_vect;
   typedef std::vector<double>                     dble_vect;
   typedef std::vector<std::pair<double, double> > band_vect;
private:
   std::string mRefChannel;      ///< Reference channel name.
   Interval    mStride;          ///< Requested dft stride [4.0]
   double      mOverlap;         ///< Requested dft overlap [0.5]
   Interval    mAverageDt;       ///< Coherence averaging interval [64s]
   double      mThreshold;       ///< Coherence trigger threshold [0.2]
   std::string mOscConfig;       ///< OSC configuration file
   std::string mOscCondition;    ///< OSC analysis condition (reset on
                                 ///  transition from off to on)
   std::string mOscGateCond;     ///< OSC gate transition (no reset at start)
   dble_vect   mBandList;        ///< Frequency band edges
   dble_vect   mBandMask;        ///< Masked bands [fmin, fmax]
   std::string mFrameName;       ///< Coherence frame file name template
   Interval    mFrameLength;     ///< length/alignment of coherence frames.
   std::string mCompression;     ///< Output frame compression mode.
   chan_vect   mChannels;        

   //-----------------------------------  monitor status variables
   calc_dft    mRefDft;
   containers::PSD mRefPSD;
   size_t      mRefCount;
   OperStateCondList mOscList;
   Time        mStartTime;
   Time        mAverageT0;
   Time        mCurrentTime;
   Time        mFrameStart;
   multiframe_writer mFrWrite;
   Interval    mTimeAnalysed;
   user_timer  mUserTime;
};

//======================================  inline methods
inline const std::string&
Coherence_Monitor::coherence_channel::channel(void) const {
   return mChannel;
}

inline const std::string&
Coherence_Monitor::coherence_channel::alias(void) const {
   return mAlias;
}

inline double
Coherence_Monitor::coherence_channel::fmax(void) const {
   return mFmax;
}

const Coherence&
Coherence_Monitor::coherence_channel::ref_coherence(void) const {
   return mCoherence;
}

Coherence&
Coherence_Monitor::coherence_channel::ref_coherence(void) {
   return mCoherence;
}

inline const containers::CSD&
Coherence_Monitor::coherence_channel::refCSD(void) const {
   return mCoherence.refCSD();
}

inline const containers::PSD&
Coherence_Monitor::coherence_channel::refPSD(void) const {
   return mCoherence.refXPSD();
}

Interval 
Coherence_Monitor::coherence_channel::stride(void) const {
   return mCoherence.stride();
}

std::string 
Coherence_Monitor::coherence_channel::window_type(void) const {
   return mCoherence.window_type();
}

#endif     //  Coherence_Monitor_HH
