/* -*- mode: c++; c-basic-offset: 3; -*- */
#ifndef GATING_UTILS_HH
#define GATING_UTILS_HH

#include "DVecType.hh"
#include "SegAccountant.hh"
#include "plot_pars.hh"
#include "TSeries.hh"
#include "timeline.hh"
#include <string>
#include <vector>

class Trend;
class DaccAPI;
class GDSPlot;

//===================================  Gate condition structure
class gate_cond {
public:
   typedef DVecType<double> DVectD;
public:
   gate_cond(void);
   gate_cond(const std::string& name);
   gate_cond(const gate_cond& x) = default;
   gate_cond(gate_cond&& x) = default;
   ~gate_cond(void) = default;
   size_t bitno(void) const;
   void collect(DaccAPI& dapi, bool ready);
   const TSeries& discriminator(void) const;
   void erase_until(const Time& t0);
   const std::string& gatevec(void) const;
   bool invert(void) const;
   const std::string& name(void) const;
   bool operator==(const std::string& test) const;
   void plot_threshold(GDSPlot& plot) const;
   void set_debug(int d);
   void set_gatevec(const std::string& file, size_t bit);
   void set_invert(bool inv);
   void set_name(const std::string& file);
   void set_plot(const plot_pars& pp);
   void set_thrcut(double cut);
   void set_thresh(const std::string& chan);
   void set_tvec_limit(Interval sz);
   double thrcut(void) const;
   const std::string& thresh(void) const;
   const timeline& time_line(void) const;
private:
   std::string _name;    ///< name of the gate condition.
   std::string _gatevec; ///< dq vector channel containing gate status bit
   std::string _thresh;  ///< gate value channel
   size_t      _bitno;   ///< bit number indicating gate is asserted (zero)
   bool        _invert;  ///< bit active low;
   Interval    _tvec_limit; ///< maximum threshold value TSeries length
   double      _thrcut;     ///< Nominal threshold
   int         _debug;
   TSeries     _threshval; ///< cut value history.
   timeline    _timeline;  ///< gate timeline
   plot_pars   _plot_parameters;
};
typedef std::vector<gate_cond> gate_cond_vect;

//===================================  Gate segment definition
class segment_def {
public:
   enum segment_mode
      {
       k_idle,            ///< no segments generated
       k_active,          ///< segment is on
       k_long,            ///< segments based on gate length
       k_fract_real,      ///< segments from fraction of real time
       k_fract_live,      ///< segments from fraction of live time
       k_fract_lock       ///< Fraction of time in single lock, <= avg
      };
public:
   segment_def(const std::string& name, trig::SegAccountant& act);
   segment_def(segment_def&& x) = default;
   ~segment_def(void) = default;
   Interval avg_time(void) const;
   size_t count(void) const;
   double fraction(void) const;
   const std::string& gate(void) const;
   bool invert(void) const;
   Interval max_time(void)const;
   segment_mode mode(void) const;
   static std::string mode_name(segment_mode s_mode);
   const std::string& name(void) const;
   void put_segment(const Time& t0, const Time& t1, bool state=true);
   void put_segment(const timeline::time_seg& s, bool state=true);
   void seg_gen(const timeline& gates, const timeline& live,
		const Time& t0, Interval dt);
   void set_average_time(Interval average);
   void set_gate(const std::string& name);
   void set_debug(int d);
   void set_fraction(double frac);
   void set_invert(bool inv);
   void set_mode(segment_mode s_mode);
   bool set_mode(const std::string& s_mode);
   void set_name(const std::string& name);
   void set_on_time(Interval odt);
   Interval total(void) const;
private:
   std::string          _name;        ///< segment name
   std::string          _gate_cond;   ///< gating condition name
   segment_mode         _mode;        ///< segment calculation mode.
   bool                 _invert;      ///< segment or averaging mode.
   Interval             _on_time;     ///< minimum segment time
   Interval             _avg_time;    ///< averaging time.
   double               _fraction;    ///< dead time fraction.
   int                  _debug;       ///< debug print level
   trig::SegAccountant& _acct;        ///< segment accountant reference
   Time                 _active;      ///< end-time of last active segment.
   Time                 _last;        ///< end-time of last summary segment.
   Time                 _analyzd;     ///< last time analyzed
   size_t               _count;       ///< number of non-overlapping segments.
   Interval             _total;       ///< total segment dead time
};
typedef std::vector<segment_def> segment_def_vect;

//===================================  Gate trend structure
class trend_def {
public:
   trend_def(const std::string& name, const std::string& gate);
   trend_def(const trend_def& x) = default;
   trend_def(trend_def&& x) = default;
   ~trend_def(void) = default;
   bool enable_dead(void) const;
   bool enable_discr(void) const;
   const std::string& gate(void) const;
   void set_enable_dead(bool enable);
   void set_enable_discr(bool enable);
   void set_prefix(const std::string& prefix);
   void trend_setup(Trend& trend, const std::string& chan_pfx);
   void trend_stride(const Time& t0, Interval dt, const gate_cond& gc,
		     Trend& trend);
private:
   std::string  _name;               ///< trend name
   std::string  _gate_cond;          ///< gating condition name
   std::string  _chan_prefix;        ///< trend channel prefix
   bool         _enable_dead;        ///< enable dead-time trending.
   bool         _enable_discr;       ///< enable discriminant trending.
};

typedef std::vector<trend_def> trend_def_vect;

//======================================  Inline methods.
inline size_t
gate_cond::bitno(void) const {
   return _bitno;
}

inline const TSeries&
gate_cond::discriminator(void) const {
   return _threshval;
}

inline void
gate_cond::erase_until(const Time& t0) {
   _timeline.erase_until(t0);
}

inline const std::string&
gate_cond::gatevec(void) const {
   return _gatevec;
}

inline bool
gate_cond::invert(void) const {
   return _invert;
}

inline bool
gate_cond::operator==(const std::string& test) const {
   return test == _name;
}

inline const std::string&
gate_cond::name(void) const {
   return _name;
}

inline double
gate_cond::thrcut(void) const {
   return _thrcut;
}

inline const std::string&
gate_cond::thresh(void) const {
   return _thresh;
}

inline const timeline&
gate_cond::time_line(void) const {
   return _timeline;
}

//====================================== inline segment definition methods
inline Interval
segment_def::avg_time(void) const {
   return _avg_time;
}

inline size_t
segment_def::count(void) const {
   return _count;
}

inline double
segment_def::fraction(void) const {
   return _fraction;
}

inline const std::string&
segment_def::gate(void) const {
   return _gate_cond;
}

inline bool
segment_def::invert(void) const {
   return _invert;
}

inline Interval
segment_def::max_time(void) const {
   return _on_time;
}

inline segment_def::segment_mode
segment_def::mode(void) const {
   return _mode;
}

inline const std::string&
segment_def::name(void) const {
   return _name;
}

inline Interval
segment_def::total(void) const{
   return _total;
}

//======================================  trend_def inline accessors
inline bool
trend_def::enable_dead(void) const {
   return _enable_dead;
}

inline bool
trend_def::enable_discr(void) const {
   return _enable_discr;
}

inline const std::string&
trend_def::gate(void) const {
   return _gate_cond;
}

#endif  // !defined(GATING_UTILS_HH)
